(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "./alter", "./create", "./select", "./delete", "./update", "./insert", "./command", "./expr", "./limit", "./show", "./sqlite", "./with", "./util"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("./alter"), require("./create"), require("./select"), require("./delete"), require("./update"), require("./insert"), require("./command"), require("./expr"), require("./limit"), require("./show"), require("./sqlite"), require("./with"), require("./util"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.alter, global.create, global.select, global._delete, global.update, global.insert, global.command, global.expr, global.limit, global.show, global.sqlite, global._with, global.util);
    global.union = mod.exports;
  }
})(typeof globalThis !== "undefined" ? globalThis : typeof self !== "undefined" ? self : this, function (_exports, _alter, _create, _select, _delete, _update, _insert, _command, _expr, _limit2, _show, _sqlite, _with, _util) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.bigQueryToSQL = bigQueryToSQL;
  _exports.unionToSQL = unionToSQL;
  _exports.multipleToSQL = multipleToSQL;
  const typeToSQLFn = {
    alter: _alter.alterToSQL,
    analyze: _sqlite.analyzeToSQL,
    attach: _sqlite.attachToSQL,
    create: _create.createToSQL,
    select: _select.selectToSQL,
    delete: _delete.deleteToSQL,
    update: _update.updateToSQL,
    insert: _insert.insertToSQL,
    drop: _command.commonCmdToSQL,
    truncate: _command.commonCmdToSQL,
    replace: _insert.insertToSQL,
    declare: _command.declareToSQL,
    use: _command.useToSQL,
    rename: _command.renameToSQL,
    call: _command.callToSQL,
    desc: _command.descToSQL,
    set: _command.setVarToSQL,
    lock: _command.lockUnlockToSQL,
    unlock: _command.lockUnlockToSQL,
    show: _show.showToSQL
  };

  function unionToSQL(stmt) {
    const fun = typeToSQLFn[stmt.type];
    const res = [fun(stmt)];
    const {
      _orderby,
      _limit
    } = stmt;

    while (stmt._next) {
      const unionKeyword = (stmt.union || 'union').toUpperCase();
      res.push(unionKeyword, fun(stmt._next));
      stmt = stmt._next;
    }

    res.push((0, _expr.orderOrPartitionByToSQL)(_orderby, 'order by'), (0, _limit2.limitToSQL)(_limit));
    return res.filter(_util.hasVal).join(' ');
  }

  function bigQueryToSQL(stmt) {
    const {
      with: withExpr,
      parentheses,
      select,
      orderby,
      limit
    } = stmt;
    const result = [(0, _with.withToSQL)(withExpr), parentheses && '(', unionToSQL(select), parentheses && ')']; // process with, orderby and limit

    result.push((0, _expr.orderOrPartitionByToSQL)(orderby, 'order by'), (0, _limit2.limitToSQL)(limit));
    return result.filter(val => val).join(' ');
  }

  function multipleToSQL(stmt) {
    const res = [];

    for (let i = 0, len = stmt.length; i < len; ++i) {
      let astInfo = stmt[i] && stmt[i].ast;
      if (!astInfo) astInfo = stmt[i];
      res.push(unionToSQL(astInfo));
    }

    return res.join(' ; ');
  }
});